﻿using Syncfusion.SfChart.XForms;
using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Diagnostics;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using Xamarin.Forms;


namespace SimpleSample
{
    public class Scroll : ContentPage
    {
        StackLayout layout = new StackLayout();
        ColumnSeries series;
        SfChart chart;
        int index = 0;
        ChartZoomPanBehaviorExt behavoir;
        private readonly ObservableCollection<ChartDataPoint> data = new ObservableCollection<ChartDataPoint>();

        public Scroll()
        {
            chart = new SfChart();
            chart.PrimaryAxis = new NumericalAxis { EnableAutoIntervalOnZooming = true };

            chart.PrimaryAxis.ActualRangeChanged += PrimaryAxis_ActualRangeChanged;
            chart.ZoomDelta += chart_ZoomDelta;
            chart.Scroll += chart_Scroll;

            chart.PrimaryAxis.ZoomFactor = 0.1;
            chart.SecondaryAxis = new NumericalAxis();

            LoadData();
            chart.VerticalOptions = LayoutOptions.FillAndExpand;

            behavoir = new ChartZoomPanBehaviorExt(chart.PrimaryAxis);
            behavoir.EnableDoubleTap = false;
            behavoir.ZoomMode = ZoomMode.X;
            chart.ChartBehaviors.Add(behavoir);

            series = new ColumnSeries();

            series.EnableDataPointSelection = true;
            series.SelectedDataPointIndex = 2;
            series.ItemsSource = data;                    

            chart.Series.Add(series);

            layout.Children.Add(chart);
            Content = layout;
            zoomFactor = chart.PrimaryAxis.ZoomFactor;
        }

        void chart_Scroll(object sender, ChartScrollEventArgs e)
        {
            if (e.Axis == chart.PrimaryAxis)
            {
                if (zoomFactor < 1)
                {
                    zoomPosition = e.ZoomPosition;

                    var delta = (actualMaximum - actualMinimum);
                    var start = actualMinimum + zoomPosition * delta;

                    var end = start + zoomFactor * delta;

                    if (start < actualMinimum)
                    {
                        end = end + (actualMinimum - start);
                        start = actualMinimum;
                    }
                    if (end > actualMaximum)
                    {
                        start = start - (end - actualMaximum);
                        end = actualMaximum;
                    }
                    behavoir.Start = start;
                    behavoir.End = end;

                    series.SelectedDataPointIndex = (int)Math.Round(start + (end - start) / 2);
                }
            }
        }

        void chart_ZoomDelta(object sender, ChartZoomDeltaEventArgs e)
        {
            if (e.Axis == chart.PrimaryAxis)
            {
                var axis = chart.PrimaryAxis;
                zoomFactor = e.CurrentZoomFactor;
                zoomPosition = e.CurrentZoomPosition;
            }
        }

        double actualMinimum, actualMaximum;
        double zoomPosition, zoomFactor;

        void PrimaryAxis_ActualRangeChanged(object sender, ActualRangeChangedEventArgs e)
        {
            actualMinimum = (double)e.ActualMinimum;
            actualMaximum = (double)e.ActualMaximum;
        }

        private void LoadData()
        {
            Random random = new Random();

            for (var i = 0; i < 50; i++)
            {
                data.Add(new ChartDataPoint(i, random.Next(10, 100)));
                index++;
            }
            index = data.Count;
        }
    }

    public class ChartZoomPanBehaviorExt : ChartZoomPanBehavior
    {
        public double Start { get; set; }

        public double End { get; set; }

        ChartAxis axis;
      
        public ChartZoomPanBehaviorExt(ChartAxis axis)
        {
            this.axis = axis;
        }

        protected override void OnTouchUp(float x, float y)
        {
            base.OnTouchUp(x, y);
            var delta = End - Start;
            var center = Start + delta / 2;
            var index = (int)Math.Round(center);
            Chart.Series[0].SelectedDataPointIndex = index;

            var diff = index - center;

            var actualStart = Start + diff;
            var actualEnd = End + diff;
            ZoomByRange(axis, actualStart, actualEnd);
        }
    }
}